import { Model } from "../../model";
export class LayoutProvider extends Model {
    constructor(attrs) {
        super(attrs);
    }
}
LayoutProvider.__name__ = "LayoutProvider";
//# sourceMappingURL=layout_provider.js.map